using System;
using System.Runtime.Serialization;

namespace DbSharp.Framework.Common
{
	/// <summary>
	/// Any exception that occurs in this software.
	/// </summary>
	/// <remarks>
    /// Exceptions that occur in this software.
	/// </remarks>
	[Serializable]
	public class DbSharpException : ApplicationException
	{
		/// <summary>
        /// Initializes a new instance of the <see cref="DbSharpException"/> class.
		/// </summary>
        public DbSharpException()
            : base("DbSharpException exception occurred.")
		{
		}

		/// <summary>
        /// Initializes a new instance of the <see cref="DbSharpException"/> class.
		/// </summary>
		/// <param name="message">The message that describes the error. </param>
		public DbSharpException(string message) : base(message)
		{
		}

		/// <summary>
        /// Initializes a new instance of the <see cref="DbSharpException"/> class.
		/// </summary>
		/// <param name="innerException">
		/// The exception that is the cause of the current exception. If the innerException parameter 
		/// is not a null reference, the current exception is raised in a catch block that handles 
		/// the inner exception.
		/// </param>
		public DbSharpException(Exception innerException) : base(innerException.Message, innerException)
		{
		}

		/// <summary>
        /// Initializes a new instance of the <see cref="DbSharpException"/> class.
		/// </summary>
		/// <param name="message">The message that describes the error. </param>
		/// <param name="innerException">
		/// The exception that is the cause of the current exception. If the innerException parameter 
		/// is not a null reference, the current exception is raised in a catch block that handles 
		/// the inner exception.
		/// </param>
		public DbSharpException(string message, Exception innerException) : base(message, innerException)
		{
		}

		/// <summary>
        /// Initializes a new instance of the <see cref="DbSharpException"/> class 
		/// with serialized data.
		/// </summary>
		/// <param name="info">
		/// The <see cref="SerializationInfo"/> that holds the serialized object 
		/// data about the exception being thrown.
		/// </param>
		/// <param name="context">
		/// The <see cref="StreamingContext"/> that contains contextual information about the source or destination.
		/// </param>
		protected DbSharpException(SerializationInfo info, StreamingContext context) : base(info, context)
		{
		}
	}
}