﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data.Common;
using Npgsql;

namespace DbSharp.Tools.DbConnectionBuilder.Core.Plugin
{
    class PostgresqlConnectionPlugin : IDbConnectionPlugin
    {
        #region IDbConnectionPlugin Members

        public PluginNamePair PluginNamePair
        {
            get { return new PluginNamePair("Npgsql") { DisplayName = "Postgresql Driver" }; }
        }

        public DbConnectionStringBuilder CreateConnectionStrBuilder(DbServerCredential credential)
        {
            //HOST=localhost;PORT=5432;PROTOCOL=3;DATABASE=postgres;USER ID=postgres;PASSWORD=drothy410;SSL=False;SSLMODE=Disable;TIMEOUT=15;SEARCHPATH=;POOLING=True;CONNECTIONLIFETIME=15;MINPOOLSIZE=1;MAXPOOLSIZE=20;SYNCNOTIFICATION=False;COMMANDTIMEOUT=20;ENLIST=False;PRELOADREADER=False;USEEXTENDEDTYPES=False;INTEGRATED SECURITY=False;COMPATIBLE=2.0.8.0
            var builder = new NpgsqlConnectionStringBuilder();
            builder.Database = credential.Database;
            builder.UserName = credential.UserId;
            builder.Password = credential.Password;
            builder.Host = credential.DataSource;
            int port;
            if (int.TryParse(credential.ServerPort, out port))
                builder.Port = port;

            builder.IntegratedSecurity = false;

            return builder;
        }

        public DbServerCredential CreateCredential(string connectionString)
        {
            DbServerCredential credential = new DbServerCredential();
            credential.UserIdEnabled = true;
            credential.PasswordEnabled = true;
            credential.DataSourceEnabled = true;
            credential.DatabaseEnabled = true;
            credential.ServerPortEnabled = true;

            var builder = new NpgsqlConnectionStringBuilder(connectionString);
            object userId;
            if (builder.TryGetValue("User Id", out userId))
            {
                credential.UserId = userId.ToString();
            }
            object pwd;
            if (builder.TryGetValue("Password", out pwd))
            {
                credential.Password = pwd.ToString();
            }
            object dataSource;
            if (builder.TryGetValue("HOST", out dataSource))
            {
                credential.DataSource = dataSource.ToString();
            }
            object database;
            if (builder.TryGetValue("database", out database))
            {
                credential.Database = database.ToString();
            }
            object serverPort;
            if (builder.TryGetValue("Port", out serverPort))
            {
                credential.ServerPort = serverPort.ToString();
            }
            
            credential.AdditionalInfo = "";

            return credential;
        }

        public DbConnection CreateDbConnection(string connectionString)
        {
            return new NpgsqlConnection(connectionString);
        }

        public List<FileVersionPair> GetCandidateDbFileVersionPairs()
        {
            var list = new List<FileVersionPair>();
            return list;
        }

        #endregion
    }
}
